import { Request, Response } from "express";
import httpStatus from "http-status";
import {
  createCustomerService,
  deleteOneCustomerService,
  getAllCustomersService,
  getOneCustomerService,
  updateOneCustomerService,
} from "../services/customer.service";
import { domainURL } from "../constants";

export const getAllCustomers = async (req: Request, res: Response) => {
  try {
    const customers = await getAllCustomersService();

    if (customers.length > 0) {
      const response = {
        totalNumberOfCustomers: customers.length,
        customers: customers.map((customer) => {
          return {
            _id: customer._id,
            name: customer.name,
            email: customer.email,
            contactNumber: customer.contactNumber,
            request: {
              type: "GET",
              url: domainURL + "managment_api/customer/" + customer._id,
            },
          };
        }),
      };
      res.status(httpStatus.OK).json(response);
    } else {
      res.status(httpStatus.NO_CONTENT).json();
    }
  } catch (error) {
    res
      .status(httpStatus.INTERNAL_SERVER_ERROR)
      .json({ message: error.message });
  }
};

export const createCustomer = async (req: Request, res: Response) => {
  try {
    const customer = await createCustomerService(req.body);
    res.status(httpStatus.CREATED).json({
      message: "Created customer successfully",
      createdCustomer: {
        _id: customer._id,
        name: customer.name,
        email: customer.email,
        contactNumber: customer.contactNumber,
        request: {
          type: "GET",
          url: domainURL + "managment_api/customer/" + customer._id,
        },
      },
    });
  } catch (error) {
    res.status(httpStatus.BAD_REQUEST).json({ message: error.message });
  }
};

export const getOneCustomerById = async (req: Request, res: Response) => {
  try {
    const customer = await getOneCustomerService(req.params.customerId);

    if (!customer) {
      res.status(httpStatus.NOT_FOUND).json({
        message: `Customer with ID: ${req.params.customerId} not found`,
      });
    } else {
      res.status(httpStatus.OK).json({
        customer,
        request: {
          type: "GET",
          description: "GET_ALL_CUSTOMERS",
          url: domainURL + "managment_api/customer/",
        },
      });
    }
  } catch (error) {
    res
      .status(httpStatus.INTERNAL_SERVER_ERROR)
      .json({ message: error.message });
  }
};

export const patchOneCustomerById = async (req: Request, res: Response) => {
  try {
    const customer = await updateOneCustomerService(
      req.params.customerId,
      req.body
    );
    if (!customer) {
      res.status(httpStatus.NOT_FOUND).json({
        message: `Customer with ID: ${req.params.customerId} not found`,
      });
    } else {
      res.status(httpStatus.OK).json({
        message: `Customer with ID:${req.params.customerId} is successfully updated`,
        updatedCustomer: customer,
        request: {
          type: "GET",
          url: domainURL + "managment_api/customer/" + customer._id,
        },
      });
    }
  } catch (error) {
    res
      .status(httpStatus.INTERNAL_SERVER_ERROR)
      .json({ message: error.message });
  }
};

export const deleteOneCustomerById = async (req: Request, res: Response) => {
  try {
    const customer = await deleteOneCustomerService(req.params.customerId);

    if (!customer) {
      res.status(httpStatus.NOT_FOUND).json({
        message: `Customer with ID: ${req.params.customerId} not found`,
      });
    } else {
      res.status(httpStatus.OK).json({
        message: `Customer with ID: ${req.params.customerId} is successfully deleted`,
        request: {
          type: "POST",
          description: "CREATE_NEW_CUSTOMER",
          url: domainURL + "managment_api/customer/",
          body: { name: "String", email: "String", contactNumber: "String" },
        },
      });
    }
  } catch (error) {
    res
      .status(httpStatus.INTERNAL_SERVER_ERROR)
      .json({ message: error.message });
  }
};
