import chai, { expect } from "chai";
import httpStatus from "http-status";
import chaiHttp from "chai-http";
import request from "supertest";
import app from "../app";

chai.use(chaiHttp);

let createdCustomerId;

describe("API TESTS /managment_api/customer", () => {
  it("should return the list of customers", (done) => {
    request(app)
      .get("/managment_api/customer")
      .end((err, res) => {
        expect(res).to.have.status(httpStatus.OK);
        expect(res.body).to.be.an("object");
        expect(res.body).to.have.property("totalNumberOfCustomers");
        expect(res.body).to.have.property("customers");
        expect(res.body.customers).to.be.an("array");
        done();
      });
  });

  it("should have a valid customer and request object for each customer", (done) => {
    request(app)
      .get("/managment_api/customer")
      .end((err, res) => {
        expect(res).to.have.status(httpStatus.OK);
        expect(res.body).to.be.an("object");
        expect(res.body).to.have.property("customers");
        expect(res.body.customers).to.be.an("array");

        res.body.customers.forEach((customer) => {
          expect(customer).to.have.property("_id");
          expect(customer).to.have.property("name");
          expect(customer).to.have.property("email");
          expect(customer).to.have.property("contactNumber");
          expect(customer).to.have.property("request");
          expect(customer.request).to.be.an("object");
          expect(customer.request).to.have.property("type");
          expect(customer.request).to.have.property("url");
        });
        done();
      });
  });

  it("should create a new customer", (done) => {
    const newCustomer = {
      name: "Test Pandzic",
      email: "test.pandzic@test.com",
      contactNumber: "0021364879",
    };
    request(app)
      .post("/managment_api/customer")
      .send(newCustomer)
      .end((err, res) => {
        expect(res).to.have.status(httpStatus.CREATED);
        expect(res.body).to.be.an("object");
        expect(res.body).to.have.property("message");
        expect(res.body).to.have.property("createdCustomer");
        expect(res.body.createdCustomer).to.have.property("_id");

        createdCustomerId = res.body.createdCustomer._id;

        expect(res.body.createdCustomer.name).to.equal(newCustomer.name);
        expect(res.body.createdCustomer.email).to.equal(newCustomer.email);
        expect(res.body.createdCustomer.contactNumber).to.equal(
          newCustomer.contactNumber
        );
        expect(res.body.createdCustomer).to.have.property("request");
        expect(res.body.createdCustomer.request).to.be.an("object");
        expect(res.body.createdCustomer.request).to.have.property("type");
        expect(res.body.createdCustomer.request).to.have.property("url");
        done();
      });
  });

  it("should get a customer by ID that created in previous test", (done) => {
    request(app)
      .get(`/managment_api/customer/${createdCustomerId}`)
      .end((err, res) => {
        expect(res).to.have.status(httpStatus.OK);
        expect(res.body).to.be.an("object");
        expect(res.body).to.have.property("customer");
        expect(res.body.customer).to.be.an("object");
        expect(res.body.customer).to.have.property("_id");
        expect(res.body.customer).to.have.property("name");
        expect(res.body.customer).to.have.property("email");
        expect(res.body.customer).to.have.property("contactNumber");
        expect(res.body).to.have.property("request");
        expect(res.body.request).to.be.an("object");
        expect(res.body.request).to.have.property("type");
        expect(res.body.request).to.have.property("description");
        expect(res.body.request).to.have.property("url");
        done();
      });
  });

  it("should update a customer by ID", (done) => {
    const updatedCustomer = {
      name: "Updated Customer Name",
      email: "updated.email@example.com",
      contactNumber: "9876543210",
    };
    request(app)
      .patch(`/managment_api/customer/${createdCustomerId}`)
      .send(updatedCustomer)
      .end((err, res) => {
        expect(res).to.have.status(httpStatus.OK);
        expect(res.body).to.be.an("object");
        expect(res.body).to.have.property("message");
        expect(res.body.message).to.be.a("string");
        expect(res.body).to.have.property("updatedCustomer");
        expect(res.body.updatedCustomer).to.be.an("object");
        expect(res.body.updatedCustomer).to.have.property("_id");
        expect(res.body.updatedCustomer).to.have.property("name");
        expect(res.body.updatedCustomer).to.have.property("email");
        expect(res.body.updatedCustomer).to.have.property("contactNumber");
        expect(res.body.updatedCustomer.name).to.equal(updatedCustomer.name);
        expect(res.body.updatedCustomer.email).to.equal(updatedCustomer.email);
        expect(res.body.updatedCustomer.contactNumber).to.equal(
          updatedCustomer.contactNumber
        );
        expect(res.body).to.have.property("request");
        expect(res.body.request).to.be.an("object");
        expect(res.body.request).to.have.property("type");
        expect(res.body.request).to.have.property("url");
        done();
      });
  });

  it("should delete a customer created in previous test by ID", (done) => {
    request(app)
      .delete(`/managment_api/customer/${createdCustomerId}`)
      .end((err, res) => {
        expect(res).to.have.status(httpStatus.OK);
        expect(res.body).to.be.an("object");
        expect(res.body).to.have.property("message");
        expect(res.body.message).to.be.a("string");
        expect(res.body).to.have.property("request");
        expect(res.body.request).to.be.an("object");
        expect(res.body.request).to.have.property("type");
        expect(res.body.request).to.have.property("description");
        expect(res.body.request).to.have.property("url");
        expect(res.body.request).to.have.property("body");
        expect(res.body.request.body).to.be.an("object");
        expect(res.body.request.body).to.have.property("name");
        expect(res.body.request.body).to.have.property("email");
        expect(res.body.request.body).to.have.property("contactNumber");
        done();
      });
  });
});
